@@ -2,7 +2,7 @@
 /******************************************************************************
 * MODULE     : r_language.cpp
 * DESCRIPTION: the "r" language
-* COPYRIGHT  : (C) 2008  Francis Jamet
+* COPYRIGHT  : (C) 2008-2019  Francis Jamet, Darcy Shen
 *******************************************************************************
 * This software falls under the GNU general public license version 3 or later.
 * It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
@@ -18,12 +18,10 @@
 
 static void parse_string (string s, int& pos);
 static bool is_in_str( char c, const char *str )  ;
-static bool is_number_start( char c ) ;
-//static inline bool is_identifier_start( char c ) ;
 static void advance_till( string s, int & pos, char c) ;
 
 r_language_rep::r_language_rep (string name):
-  abstract_language_rep (name), colored ("")
+  abstract_language_rep (name)
 { 
   eval ("(use-modules (utils misc tm-keywords))");
   list<string> l= as_list_string (eval ("(map symbol->string highlight-any)"));
@@ -32,33 +30,25 @@ r_language_rep::r_language_rep (string name):
     l= l->next;
   }
   number_parser.use_r_style ();
+
+  array<char> start_chars, extra_chars;
+  start_chars << '.';
+  extra_chars << '.' << '_';
+  identifier_parser.set_start_chars (start_chars);
+  identifier_parser.set_extra_chars (extra_chars);
 }
 
 text_property
 r_language_rep::advance (tree t, int& pos) {
   string s= t->label;
+  if (pos==N(s)) return &tp_normal_rep;
 
-  if (pos==N(s)) {  return &tp_normal_rep; }
-
-  char c= s[pos];
-
-  if (c == ' ') {
-    pos++; return &tp_space_rep; 
-  } 
-
-  if ( is_number_start(c) ) {
-    int opos =pos ;
-    number_parser.parse (s, pos); 
-    if( opos != pos )
-      return &tp_normal_rep; 
-  } 
-
-  if (is_alpha (c) || is_in_str (c, "_.")
-      //|| (c == '$') // For some reason, when this is uncommented, TeXmacs gets stuck on entering $.
-      ) {
-    parse_alpha (s, pos); 
+  if (blanks_parser.parse (s, pos))
+    return &tp_space_rep;
+  if (number_parser.parse (s, pos))
+    return &tp_normal_rep;
+  if (identifier_parser.parse (s, pos))
     return &tp_normal_rep; 
-  } 
 
   tm_char_forwards (s, pos);
   return &tp_normal_rep;
@@ -166,11 +156,6 @@ r_language_rep::belongs_to_identifier (char c) {
 	   is_in_str( c, "_." ) ) ;
 }
 
-static inline bool 
-is_number_start( char c ) {
-  return( is_digit(c) || is_in_str( c, "." ) ) ;
-}
-
 /*
 static inline bool 
 is_identifier_start( char c ) {
@@ -364,15 +349,6 @@ is_in_str( char c, const char *str ) {
   return false ;
 }
 
-
-static void
-parse_comment_single_line (string s, int& pos) {
-  if (pos>=N(s)) return;
-  if (s[pos]!='#') return;
-  pos=N(s);	
-}
-
-
 static void
 parse_parenthesized (string s, int& pos) {
   int i=pos;
@@ -447,10 +423,9 @@ r_language_rep::get_color (tree t, int start, int end) {
       parse_string (s, pos);
       if (opos<pos) break;
 
-      parse_comment_single_line (s, pos);
-      if (opos < pos) {
-	type= "comment";
-	break;
+      if (inline_comment_parser.parse (s, pos)) {
+        type= "comment";
+        break;
       }
 
       pos++;
@@ -468,8 +443,7 @@ r_language_rep::get_color (tree t, int start, int end) {
       possible_class= possible_future_class;
       opos= pos;
 
-      parse_blanks (s, pos);
-      if (opos<pos) break;
+      if (blanks_parser.parse (s, pos)) break;
 
       parse_string (s, pos);
       if (opos<pos) {
@@ -483,8 +457,7 @@ r_language_rep::get_color (tree t, int start, int end) {
         break;
       }
 
-      number_parser.parse (s, pos);
-      if (opos<pos) {
+      if (number_parser.parse (s, pos)) {
         type= "number";
         backquote= false;
         postfix= false;
@@ -493,8 +466,7 @@ r_language_rep::get_color (tree t, int start, int end) {
         break;
       }
 
-      parse_comment_single_line (s, pos);
-      if (opos<pos) {
+      if (inline_comment_parser.parse (s, pos)) {
         type= "comment";
         backquote= false;
         postfix= false;
@@ -666,20 +638,22 @@ r_language_rep::get_color (tree t, int start, int end) {
     do {
       do {
         opos=pos;
-        parse_blanks (s, pos);
-        if (opos<pos) break;
+        if (blanks_parser.parse (s, pos)) break;
 
         parse_identifier (colored, s, pos);
         if (opos<pos) { possible_function= true; break; }
 
-        number_parser.parse (s, pos);
-        if (opos<pos) { possible_function= true; break; }
+        if (number_parser.parse (s, pos)) {
+          possible_function= true;
+          break;
+        }
 
         parse_constant (colored, s, pos);
         if (opos<pos) { possible_function= true; break; }
 
-        parse_comment_single_line (s, pos);
-        if (opos<pos) break;
+        if (inline_comment_parser.parse (s, pos)) {
+          break;
+        }
 
         parse_parenthesized (s, pos);
         if (opos<pos) { possible_function= true; break; }
@@ -692,16 +666,16 @@ r_language_rep::get_color (tree t, int start, int end) {
     } else do {
       do {
         opos=pos;
-        parse_blanks (s, pos);
-        if (opos<pos) break;
+        if (blanks_parser.parse (s, pos)) break;
         parse_identifier (colored, s, pos);
         if (opos<pos) break;
         number_parser.parse (s, pos);
         if (opos<pos) break;
         parse_constant (colored, s, pos);
         if (opos<pos) break;
-        parse_comment_single_line(s, pos);
-        if (opos<pos) break;
+        if (inline_comment_parser.parse (s, pos)) {
+          break;
+        }
         parse_parenthesized (s, pos);
         if (opos<pos) break;
 
@@ -716,20 +690,21 @@ r_language_rep::get_color (tree t, int start, int end) {
     do {
       do {
         opos=pos;
-        parse_blanks (s, pos);
-        if (opos<pos) break;
+        if (blanks_parser.parse (s, pos)) break;
 
         parse_identifier (colored, s, pos);
         if (opos<pos) break;
 
-        number_parser.parse (s, pos);
-        if (opos<pos) break;
+        if (number_parser.parse (s, pos)) {
+          break;
+        }
 
         parse_constant (colored, s, pos);
         if (opos<pos) break;
 
-        parse_comment_single_line(s, pos);
-        if (opos<pos) break;
+        if (inline_comment_parser.parse (s, pos)) {
+          break;
+        }
 
         parse_parenthesized (s, pos);
         if (opos<pos) break;
